{ lib
, pkgs
, fetchFromGitHub
, buildUBoot
, buildPackages
, stdenv
}:

let
  bl31Blobs = fetchFromGitHub {
    owner = "rockchip-linux";
    repo = "rkbin";
    rev = "87885c48747a358dc8eb1808db99e313935e129e";
    sha256 = "aQj4JHk67dlr9OGI5SdbQSypQllIasF0TZFoPNARuTo=";
  };

  rkbinBlobs = fetchFromGitHub {
    owner = "rockchip-linux";
    repo = "rkbin";
    rev = "a4c6de9ea29f275bb1d08c94ccded51ff2ab5b92";
    sha256 = "e3RH/4hi+xc5cQpzt2txyZYNMGQvC1jGDJpzBY2QSHo=";
  };

  buildQuartz64UBoot = (defconfig: buildUBoot {
    version = "2023.04-rc1";

    defconfig = defconfig;

    src = fetchFromGitHub {
      owner = "CounterPillow";
      repo = "u-boot-quartz64";
      rev = "be645fef058885e2fc9882e839dacd6941693ac6";
      sha256 = "icoeVb9WGDRcQEzym9yd57sWX8PGYx2ufN3Dj3IT/2c=";
    };

    nativeBuildInputs = with pkgs; [
      ncurses # tools/kwboot
      bc
      bison
      dtc
      flex
      openssl
      (buildPackages.python3.withPackages (p: [
        p.libfdt
        p.setuptools # for pkg_resources
        p.pyelftools
      ]))
      swig
      which # for scripts/dtc-version.sh
    ];

    extraMakeFlags = [
      "ARCH=arm"
    ];

    extraMeta = {
      platforms = [ "aarch64-linux" ];
      license = lib.licenses.unfreeRedistributableFirmware;
    };

    filesToInstall = [ "u-boot.itb" "idbloader.img" ];

    BL31 = "${bl31Blobs}/bin/rk35/rk3568_bl31_v1.34.elf";
    ROCKCHIP_TPL = "${rkbinBlobs}/bin/rk35/rk3566_ddr_1056MHz_v1.13.bin";

    preConfigure = ''
      make mrproper
    '';
  });
in
{
  ubootSoQuartzBlade = buildQuartz64UBoot "soquartz-blade-rk3566_defconfig";
  ubootSoQuartzCM4 = buildQuartz64UBoot "soquartz-cm4io-rk3566_defconfig";
  ubootSoQuartzModelA = buildQuartz64UBoot "soquartz-model-a-rk3566_defconfig";
  ubootQuartz64b = buildQuartz64UBoot "quartz64-b-rk3566_defconfig";
  ubootQuartz64a = buildQuartz64UBoot "quartz64-a-rk3566_defconfig";
}
